-- DCS - Bunker Buster script
-- Version 1.0

-- Based on iron bombing range script 
-- Tracks bombs of certain types and if they land close enough to a bunker, destroys the bunker
-- Designed for my Moving Mud campaign in order to allow iron bombs a chance to destroy bunkers
-- basically simulates a less robust bunker than is simulated in DCS
-- Changelog

-- Requires MIST 4.0.57 or newer!


range  = {} 
KillDist = 25
ExpMagnitude = 1000
trigger.action.outText( 'Bunker Busting Script Loaded - V1.0 18 07d', 10 )

-- Zone Names
range.bombingMinAlt = 0
range.bombingTargets = {

    "bunker1",
    "bunker2",
    "bunker3",
    "bunker4",
    "bunker5",
    "bunker6",
    "bunker7",
    "bunker8",
    "bunker9",
    "bunker10",
    "bunker11",
    "bunker12",
    "bunker13",
    "bunker14",
    "bunker15",

}


-- Handles all world events
range.eventHandler = {}
function range.eventHandler:onEvent(_eventDCS)

    if _eventDCS == nil or _eventDCS.initiator == nil then
        return true
    end


    local status, err = pcall(function(_event)
    
        if _event.id == world.event.S_EVENT_SHOT then

            local _weapon = _event.weapon:getTypeName()
 --           local _weaponStrArray = range.split(_weapon,"%.")
 --           local _weaponName = _weaponStrArray[#_weaponStrArray]
              -- Note It seems that DCS used to append "weapons.bombs" or "Weapons.nurs" to the weapon names
              -- but as of Jan 2021, it no longer does.  For now, it will be necessary to go through the list of
              -- bombs and rockets one at a time

            env.info("Dropping ".._weapon)
   
            -- trigger.action.outText( 'Bomb Drop '.._event.weapon:getTypeName(), 10 )
            if _weapon == "Mk_83" -- Mk_83
              or _weapon == "Mk_82" -- Mk_82
              or _weapon == "MK_82SNAKEYE"
              --or _weapon == "ROCKEYE"-- Rockeye
              --or _weapon == "HYDRA_70_MK5"-- 2.75" Rockets
              --or _weapon == "HYDRA_70_M151"-- 2.75" Rockets
              --or _weapon == "FFAR Mk1 HE" -- 2.75" Rockets
              --or _weapon == "Zuni_127" -- Zuni Rockets
              --or _weapon == "FAB_250"-- FAB 250 Russian bomb
              --or _weapon == "C_13"-- 130mm Russian rocket
              --or _weapon == "C_8"-- 80mm Russian rocket   
              --or _weapon == "C_24"-- S24 Russian Rocket
              --or _weapon == "BDU_33"-- Practice Bomb
              
				    and _event.initiator:getPosition().p.y > range.bombingMinAlt
            then
                local _ordnance =  _event.weapon
                env.info("Tracking ".._weapon.." - ".._ordnance:getName())
                -- trigger.action.outText( "Tracking ".._weapon, 10 )
                local _lastBombPos = {x=0,y=0,z=0}
                local _BoomPos = {x=0,y=0,z=0}

                local _unitName = _event.initiator:getName()
                local trackBomb = function(_previousPos)

                    local _unit = Unit.getByName(_unitName)
                    --    env.info("Checking...")
                    if _unit ~= nil and _unit:getPlayerName() ~= nil then

                        -- when the pcall returns a failure the weapon has hit
                        local _status,_bombPos =  pcall(function()
                            -- env.info("protected")
                            return _ordnance:getPoint()
                        end)

                        if  _status then
                            --ok! still in the air
                            _lastBombPos = {x = _bombPos.x, y = _bombPos.y, z= _bombPos.z }

                            return timer.getTime() + 0.005 -- check again !
                        else
                            --hit
                            -- get closest target to last position
                            local _closetTarget = nil
                            local _distance = nil
                            local _direction = nil

                            for _,_targetZone in pairs(range.bombingTargets) do

                                local _temp = range.getDistance(_targetZone.point, _lastBombPos)

                                if _distance == nil or _temp < _distance then
                                    _distance = _temp
                                    _closetTarget = _targetZone
                                    _direction = range.getDirection(_targetZone.point, _lastBombPos)
                                end
                            end

                            env.info(_distance.." from ".._closetTarget.name.." @".._direction)

                            if _distance < KillDist then
                              if _closetTarget.name == "bunker1" then 
                                trigger.action.setUserFlag("201",1)
                              end
                              if _closetTarget.name == "bunker2" then 
                                trigger.action.setUserFlag("202",1)
                              end
                              if _closetTarget.name == "bunker3" then 
                                trigger.action.setUserFlag("203",1)
                              end                              
                              if _closetTarget.name == "bunker4" then 
                                trigger.action.setUserFlag("204",1)
                              end
                              if _closetTarget.name == "bunker5" then 
                                trigger.action.setUserFlag("205",1)
                              end                              
                                 --trigger.action.outText(_weapon.._DistResult.._closetTarget.name.._DirResult, 10,10,false)
                                 

                            end
                        end

                    end

                    return
                end

                timer.scheduleFunction(trackBomb, nil, timer.getTime() + 1)
            end
        end


        return true

    end, _eventDCS)

    if (not status) then
        env.error(string.format("Error while handling event %s", err),false)
    end
end


function range.getGroupId(_unit)

    local _unitDB =  mist.DBs.unitsById[tonumber(_unit:getID())]
    if _unitDB ~= nil and _unitDB.groupId then
        return _unitDB.groupId
    end

    return nil
end

--get distance in meters assuming a Flat world
function range.getDistance(_point1, _point2)

    local xUnit = _point1.x
    local yUnit = _point1.z
    local xZone = _point2.x
    local yZone = _point2.z

    local xDiff = xUnit - xZone
    local yDiff = yUnit - yZone

    return math.sqrt(xDiff * xDiff + yDiff * yDiff)
end
function range.getDirection(_point1, _point2)
    local tgtBearing
    local xUnit = _point1.x
    local yUnit = _point1.z
    local xZone = _point2.x
    local yZone = _point2.z
    
    local xDiff = xUnit - xZone
    local yDiff = yUnit - yZone
    
    local tgtAngle = math.deg(math.atan(yDiff/xDiff))
    
    if xDiff > 0 then 
    tgtBearing = 180 + tgtAngle 
    end
    
    if xDiff < 0 and tgtAngle > 0 then
    tgtBearing = tgtAngle 
    end
    
    if xDiff < 0 and tgtAngle < 0 then
    tgtBearing = 360 + tgtAngle
    end
    
    
 env.info("xDiff "..xDiff.." yDiff "..yDiff)    
    return tgtBearing
end
--http://stackoverflow.com/questions/1426954/split-string-in-lua
function range.split(str, sep)
    local result = {}
    local regex = ("([^%s]+)"):format(sep)
    for each in str:gmatch(regex) do
        table.insert(result, each)
    end
    return result
end

--init

local _tempTargets = range.bombingTargets

range.bombingTargets = {}

for _,_targetZone in pairs(_tempTargets) do

    local _triggerZone = trigger.misc.getZone(_targetZone)

    if _triggerZone then
        table.insert(range.bombingTargets,{name=_targetZone,point=_triggerZone.point})
        env.info("Done for: ".._targetZone)
    else
        env.info("Failed for: ".._targetZone)

    end

end


world.addEventHandler(range.eventHandler)

